@@ -1,5 +1,5 @@
 r"""
-This module contains the QCD beta function coefficients.
+Compute the QCD beta function coefficients.
 
 See :doc:`pQCD ingredients </theory/pQCD>`.
 """
@@ -7,12 +7,12 @@ See :doc:`pQCD ingredients </theory/pQCD>`.
 import numba as nb
 
 from . import constants
-from ekore.harmonics.constants import zeta3
+from .constants import zeta3
 
 
 @nb.njit(cache=True)
 def beta_qcd_as2(nf):
-    """Computes the first coefficient of the QCD beta function.
+    r"""Compute the first coefficient of the QCD beta function.
 
     Implements Eq. (3.1) of :cite:`Herzog:2017ohr`.
 
@@ -32,8 +32,8 @@ def beta_qcd_as2(nf):
 
 
 @nb.njit(cache=True)
-def beta_qed_aem2(nf):
-    """Computes the first coefficient of the QED beta function.
+def beta_qed_aem2(nf, nl):
+    r"""Compute the first coefficient of the QED beta function.
 
     Implements Eq. (7) of :cite:`Surguladze:1996hx`.
 
@@ -41,6 +41,8 @@ def beta_qed_aem2(nf):
     ----------
         nf : int
             number of active flavors
+        nl : int
+            number of leptons partecipating to alphaem running
 
     Returns
     -------
@@ -50,13 +52,15 @@ def beta_qed_aem2(nf):
     """
     nu = constants.uplike_flavors(nf)
     nd = nf - nu
-    beta_qed_aem2 = -4.0 / 3 * constants.NC * (nu * constants.eu2 + nd * constants.ed2)
+    beta_qed_aem2 = (
+        -4.0 / 3 * (nl + constants.NC * (nu * constants.eu2 + nd * constants.ed2))
+    )
     return beta_qed_aem2
 
 
 @nb.njit(cache=True)
 def beta_qcd_as3(nf):
-    """Computes the second coefficient of the QCD beta function.
+    r"""Compute the second coefficient of the QCD beta function.
 
     Implements Eq. (3.2) of :cite:`Herzog:2017ohr`.
 
@@ -80,8 +84,8 @@ def beta_qcd_as3(nf):
 
 
 @nb.njit(cache=True)
-def beta_qed_aem3(nf):
-    """Computes the second coefficient of the QED beta function.
+def beta_qed_aem3(nf, nl):
+    r"""Compute the second coefficient of the QED beta function.
 
     Implements Eq. (7) of :cite:`Surguladze:1996hx`.
 
@@ -89,6 +93,8 @@ def beta_qed_aem3(nf):
     ----------
         nf : int
             number of active flavors
+        nl : int
+            number of leptons partecipating to alphaem running
 
     Returns
     -------
@@ -98,15 +104,15 @@ def beta_qed_aem3(nf):
     """
     nu = constants.uplike_flavors(nf)
     nd = nf - nu
-    beta_qed_aem3 = (
-        -4.0 * constants.NC * (nu * constants.eu2**2 + nd * constants.ed2**2)
+    beta_qed_aem3 = -4.0 * (
+        nl + constants.NC * (nu * constants.eu2**2 + nd * constants.ed2**2)
     )
     return beta_qed_aem3
 
 
 @nb.njit(cache=True)
 def beta_qcd_as2aem1(nf):
-    """Computes the first QED correction of the QCD beta function.
+    r"""Compute the first QED correction of the QCD beta function.
 
     Implements Eq. (7) of :cite:`Surguladze:1996hx`.
 
@@ -129,7 +135,7 @@ def beta_qcd_as2aem1(nf):
 
 @nb.njit(cache=True)
 def beta_qed_aem2as1(nf):
-    """Computes the first QCD correction of the QED beta function.
+    r"""Compute the first QCD correction of the QED beta function.
 
     Implements Eq. (7) of :cite:`Surguladze:1996hx`.
 
@@ -154,7 +160,7 @@ def beta_qed_aem2as1(nf):
 
 @nb.njit(cache=True)
 def beta_qcd_as4(nf):
-    """Computes the third coefficient of the QCD beta function
+    r"""Compute the third coefficient of the QCD beta function.
 
     Implements Eq. (3.3) of :cite:`Herzog:2017ohr`.
 
@@ -183,7 +189,7 @@ def beta_qcd_as4(nf):
 
 @nb.njit(cache=True)
 def beta_qcd_as5(nf):
-    """Computes the fourth coefficient of the QCD beta function
+    r"""Compute the fourth coefficient of the QCD beta function.
 
     Implements Eq. (3.6) of :cite:`Herzog:2017ohr`.
 
@@ -210,7 +216,7 @@ def beta_qcd_as5(nf):
 
 @nb.njit(cache=True)
 def beta_qcd(k, nf):
-    """Compute value of a beta_qcd coefficients
+    r"""Compute value of a beta_qcd coefficients.
 
     Parameters
     ----------
@@ -242,8 +248,8 @@ def beta_qcd(k, nf):
 
 
 @nb.njit(cache=True)
-def beta_qed(k, nf):
-    """Compute value of a beta_qed coefficients
+def beta_qed(k, nf, nl):
+    r"""Compute value of a beta_qed coefficients.
 
     Parameters
     ----------
@@ -251,6 +257,8 @@ def beta_qed(k, nf):
             perturbative order
         nf : int
             number of active flavors
+        nl : int
+            number of leptons partecipating to alphaem running
 
     Returns
     -------
@@ -260,9 +268,9 @@ def beta_qed(k, nf):
     """
     beta_ = 0
     if k == (0, 2):
-        beta_ = beta_qed_aem2(nf)
+        beta_ = beta_qed_aem2(nf, nl)
     elif k == (0, 3):
-        beta_ = beta_qed_aem3(nf)
+        beta_ = beta_qed_aem3(nf, nl)
     elif k == (1, 2):
         beta_ = beta_qed_aem2as1(nf)
     else:
@@ -272,7 +280,7 @@ def beta_qed(k, nf):
 
 @nb.njit(cache=True)
 def b_qcd(k, nf):
-    """Compute b_qcd coefficient.
+    r"""Compute b_qcd coefficient.
 
     Parameters
     ----------
@@ -291,8 +299,8 @@ def b_qcd(k, nf):
 
 
 @nb.njit(cache=True)
-def b_qed(k, nf):
-    """Compute b_qed coefficient.
+def b_qed(k, nf, nl):
+    r"""Compute b_qed coefficient.
 
     Parameters
     ----------
@@ -300,6 +308,8 @@ def b_qed(k, nf):
             perturbative order
         nf : int
             number of active flavors
+        nl : int
+            number of leptons partecipating to alphaem running
 
     Returns
     -------
@@ -307,4 +317,4 @@ def b_qed(k, nf):
             b_qed_k(nf)
 
     """
-    return beta_qed(k, nf) / beta_qed((0, 2), nf)
+    return beta_qed(k, nf, nl) / beta_qed((0, 2), nf, nl)
