@@ -47,16 +47,24 @@ def compute_pegasus_data(theory, operators, skip_pdfs, rotate_to_evolution_basis
     if theory["Q0"] != theory["Qref"]:
         raise ValueError("Initial scale Q0 must be equal to Qref in Pegasus.")
 
-    pegasus.initevol(
-        imodev, theory["PTO"], ivfns, nf, theory["fact_to_ren_scale_ratio"] ** 2
-    )
-    pegasus.initinp(
-        theory["alphas"],
-        theory["Qref"] ** 2,
-        (theory["kcThr"] * theory["mc"]) ** 2,
-        (theory["kbThr"] * theory["mb"]) ** 2,
-        (theory["ktThr"] * theory["mt"]) ** 2,
-    )
+    if operators["polarized"]:
+        pegasus.initpol(imodev, theory["PTO"], ivfns, nf, 1.0 / theory["XIF"] ** 2)
+        pegasus.initpinp(
+            theory["alphas"],
+            theory["Qref"] ** 2,
+            (theory["kcThr"] * theory["mc"]) ** 2,
+            (theory["kbThr"] * theory["mb"]) ** 2,
+            (theory["ktThr"] * theory["mt"]) ** 2,
+        )
+    else:
+        pegasus.initevol(imodev, theory["PTO"], ivfns, nf, 1.0 / theory["XIF"] ** 2)
+        pegasus.initinp(
+            theory["alphas"],
+            theory["Qref"] ** 2,
+            (theory["kcThr"] * theory["mc"]) ** 2,
+            (theory["kbThr"] * theory["mb"]) ** 2,
+            (theory["ktThr"] * theory["mt"]) ** 2,
+        )
 
     # better return always the flavor basis and then rotate
     # if rotate_to_evolution_basis:
@@ -74,13 +82,12 @@ def compute_pegasus_data(theory, operators, skip_pdfs, rotate_to_evolution_basis
 
     # run pegaus
     out_tabs = {}
-    for q2 in operators["Q2grid"]:
-
+    for mu in operators["mugrid"]:
         tab = {}
         for x in target_xgrid:
             # last two numbers are the min and max pid to calculate,
             # keep everthing for simplicity.
-            xf, _ = pegasus.xparton(x, q2, -6, 6)
+            xf, _ = pegasus.xparton(x, mu**2, -6, 6)
             temp = dict(zip(labels, xf))
             for pid in labels:
                 if pid in skip_pdfs:
@@ -91,16 +98,23 @@ def compute_pegasus_data(theory, operators, skip_pdfs, rotate_to_evolution_basis
 
         # rotate if needed
         if rotate_to_evolution_basis:
+            qed = theory["QED"] > 0
+            if not qed:
+                evol_basis = br.evol_basis
+                rotate_flavor_to_evolution = br.rotate_flavor_to_evolution
+            else:
+                evol_basis = br.unified_evol_basis
+                rotate_flavor_to_evolution = br.rotate_flavor_to_unified_evolution
             pdfs = np.array(
                 [
                     tab[pid] if pid in tab else np.zeros(len(target_xgrid))
                     for pid in br.flavor_basis_pids
                 ]
             )
-            evol_pdf = br.rotate_flavor_to_evolution @ pdfs
-            tab = dict(zip(br.evol_basis, evol_pdf))
+            evol_pdf = rotate_flavor_to_evolution @ pdfs
+            tab = dict(zip(evol_basis, evol_pdf))
 
-        out_tabs[q2] = tab
+        out_tabs[mu**2] = tab
 
     ref = {"target_xgrid": target_xgrid, "values": out_tabs}
 
