@@ -1,10 +1,14 @@
+"""The unpolarized, space-like |N3LO| quark-gluon |OME|."""
 import numba as nb
 import numpy as np
 
+from .....harmonics import cache as c
+
 
 @nb.njit(cache=True)
-def A_qg(n, sx, nf, L):
-    r"""Computes the |N3LO| singlet |OME| :math:`A_{qg}^{S,(3)}(N)`.
+def A_qg(n, cache, nf, L):
+    r"""Compute the |N3LO| singlet |OME| :math:`A_{qg}^{S,(3)}(N)`.
+
     The expression is presented in :cite:`Bierenbaum:2009mv`.
 
     When using the code, please cite the complete list of references
@@ -14,8 +18,8 @@ def A_qg(n, sx, nf, L):
     ----------
     n : complex
         Mellin moment
-    sx : list
-        harmonic sums cache
+    cache: numpy.ndarray
+        Harmonic sum cache
     nf : int
         number of active flavor below the threshold
     L : float
@@ -27,10 +31,16 @@ def A_qg(n, sx, nf, L):
         :math:`A_{qg}^{S,(3)}(N)`
 
     """
-    S1, _ = sx[0]
-    S2, Sm2 = sx[1]
-    S3, S21, _, _, _, Sm3 = sx[2]
-    S4, S31, S211, _, _, _, Sm4 = sx[3]
+    S1 = c.get(c.S1, cache, n)
+    S2 = c.get(c.S2, cache, n)
+    Sm2 = c.get(c.Sm2, cache, n, is_singlet=True)
+    S3 = c.get(c.S3, cache, n)
+    S21 = c.get(c.S21, cache, n)
+    Sm3 = c.get(c.Sm3, cache, n, is_singlet=True)
+    S4 = c.get(c.S4, cache, n)
+    S31 = c.get(c.S31, cache, n)
+    S211 = c.get(c.S211, cache, n)
+    Sm4 = c.get(c.Sm4, cache, n, is_singlet=True)
     a_qg_l0 = 0.3333333333333333 * nf * (
         (
             -8.547960200246003
